# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

* [`algosec::agent`](#algosecagent): This resource manages the `resource_api::agent` and the algosec-sdk gem on an agent.
* [`algosec::server`](#algosecserver): This resource manages the `resource_api::server` on the server.

**Resource types**

* [`algosec_application`](#algosec_application): This type provides Puppet with the capabilities to manage Applications on AlgoSec BusinessFlow. Currently the management capabilities of this
* [`algosec_apply_draft`](#algosec_apply_draft): When evaluated, this resource apply all outstanding application drafts in the managed AlgoSec server.
* [`algosec_flow`](#algosec_flow): This type provides Puppet with the capabilities to manage "Application Flows" on AlgoSec BusinessFlow. The usage of this resources is depende

**Tasks**

* [`apply_drafts`](#apply_drafts): Apply application drafts for all managed applications.

## Classes

### algosec::agent

This resource manages the `resource_api::agent` and the algosec-sdk gem on an agent.

#### Examples

##### 

```puppet
include algosec::agent
```

### algosec::server

This resource manages the `resource_api::server` on the server.

#### Examples

##### 

```puppet
include algosec::server
```

## Resource types

### algosec_application

This type provides Puppet with the capabilities to manage Applications on AlgoSec BusinessFlow.
Currently the management capabilities of this resource are limited until proper update API methods are
implemented in AlgoSec BusinessFlow.

#### Properties

The following properties are available in the `algosec_application` type.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this application should be present or absent on the target AlgoSec BusinessFlow.

Default value: present

#### Parameters

The following parameters are available in the `algosec_application` type.

##### `name`

namevar

Data type: `String[2]`

The name of the AlgoSec BusinessFlow application.

### algosec_apply_draft

When evaluated, this resource apply all outstanding application drafts in the managed AlgoSec server.

* **Note** If managed applications are defined in the device config, only their drafts will be applied.

#### Properties

The following properties are available in the `algosec_apply_draft` type.

##### `apply`

Data type: `Boolean`

Whether an `apply application draft`should happen

#### Parameters

The following parameters are available in the `algosec_apply_draft` type.

##### `name`

namevar

Data type: `Enum["apply"]`

The name of the resource you want to manage. Can only be "apply".

### algosec_flow

This type provides Puppet with the capabilities to manage "Application Flows" on AlgoSec BusinessFlow.
The usage of this resources is dependent upon the resource deceleration of AlgoSec BusinessFlow Application.
Please see how-to-use examples and the algosec_application resource.

#### Properties

The following properties are available in the `algosec_flow` type.

##### `sources`

Data type: `Array[String[1],1]`

List of IPs or ABF network objects of traffic sources for the application flow.

##### `destinations`

Data type: `Array[String[1],1]`

List of IPs or ABF network objects of traffic destinations for the application flow.

##### `services`

Data type: `Array[String[1],1]`

List of traffic services to allow in the flow. Services can be as defined on AlgoSec
BusinessFlow or in a proto/port format (only UDP and TCP are supported as proto. e.g. tcp/50)

##### `users`

Data type: `Optional[Array[String[1]]]`

List of users which the application flow is relevant to.

##### `applications`

Data type: `Optional[Array[String[1]]]`

List of network application names which the application flow is relevant to.

##### `comment`

Data type: `Optional[String]`

Optional comment to attach to the flow.

##### `ensure`

Data type: `Enum[present, absent]`

Whether this resource should be present or absent on the target system.

Default value: present

#### Parameters

The following parameters are available in the `algosec_flow` type.

##### `name`

namevar

Data type: `String`

The name of the application flow.

##### `application`

namevar

Data type: `String`

The name of the application that the flow belongs to.

## Tasks

### apply_drafts

Apply application drafts for all managed applications.

**Supports noop?** false

#### Parameters

##### `credentials_file`

Data type: `String`

The filename of the credentials file (as referenced in device.conf)

